#!/usr/bin/env python
# -*- coding:utf-8 -*-
'''
@Description: 
@Author: Ecohnoch(xcy)
@Date: 2022-10-06 12:30:47
@LastEditors: Kermit
@LastEditTime: 2022-11-11 19:45:41
'''

login_url = 'https://vsource.club/core/auth/login'
