#!/usr/bin/env python
# -*- coding:utf-8 -*-
'''
@Description: 
@Author: Kermit
@Date: 2022-10-24 14:45:40
@LastEditors: Kermit
@LastEditTime: 2022-11-28 13:09:57
'''


interval = 0.3
call_timeout = 30

algo_info_url = 'https://vsource.club/core/algo/pypi/info'
storage_file_url = 'https://vsource.club/core/storage/file'
